

love.graphics.setDefaultFilter("nearest", "nearest")
love.graphics.setBackgroundColor(35, 35, 30)
love.graphics.setPointSize(4)
love.audio.setVolume(.7)

-------------------------------------------------------------------- --
-- Fonts --
-------------------------------------------------------------------- --
local font = love.graphics.newImageFont("assets/font.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890:=.,'!?-", 1)

-------------------------------------------------------------------- --
-- Spritesheet --
-------------------------------------------------------------------- --
local Anim8 = require("source.library.anim8")
local spritesheet = love.graphics.newImage("assets/spritesheet.png")
local width, height = spritesheet:getDimensions()
local FlameGrid = Anim8.newGrid(5, 3, width, height, 0, 60)
local PassengerWaveLeftGrid   = Anim8.newGrid(10, 14, width, height, 0, 32)
local PassengerWaveRightGrid  = Anim8.newGrid(10, 14, width, height, 0, 46)
local PassengerMoveLeftGrid   = Anim8.newGrid(10, 14, width, height, 90, 32)
local PassengerMoveRightGrid  = Anim8.newGrid(10, 14, width, height, 90, 46)
local birdGrid  = Anim8.newGrid(16, 10, width, height, 15, 60)
local bird      = Anim8.newAnimation(birdGrid('1-7', 1), .1)

local taxi = {
  ["Taxi"]          = love.graphics.newQuad(0, 16, 32, 15, width, height),
  ["TaxiPassenger"] = love.graphics.newQuad(32, 16, 32, 15, width, height),
  ["Flame"]         = Anim8.newAnimation(FlameGrid('1-3', 1), .1),
}
local platform = {
  ["TaxiStation"] = love.graphics.newQuad(0, 8, 64, 8, width, height),
  ["GasStation"]  = love.graphics.newQuad(64, 8, 64, 8, width, height),
}

local passenger = {
  ["WaveLeft"]   = Anim8.newAnimation(PassengerWaveLeftGrid('1-9', 1), .15),
  ["WaveRight"]  = Anim8.newAnimation(PassengerWaveRightGrid('1-9', 1), .15),
  ["MoveLeft"]   = Anim8.newAnimation(PassengerMoveLeftGrid('1-3', 1), .15),
  ["MoveRight"]  = Anim8.newAnimation(PassengerMoveRightGrid('1-3', 1), .15),
}


-------------------------------------------------------------------- --
-- Sounds --
-------------------------------------------------------------------- --
local sound = {
  ["thrust"] = "assets/sounds/thrust.ogg",
  ["noFuel"] = "assets/sounds/nofuel.ogg",
  ["refuel"] = "assets/sounds/refuel.ogg",
  ["getIn"] = "assets/sounds/getIn.ogg",
  ["getOut"] = "assets/sounds/getOut.ogg",
  ["explosion"] = "assets/sounds/explosion.ogg",
  ["heyTaxi"] = "assets/sounds/heytaxi.ogg",
  ["noo"] = "assets/sounds/noo.ogg",
  ["station1"] = "assets/sounds/station1.ogg",
  ["station2"] = "assets/sounds/station2.ogg",
  ["station3"] = "assets/sounds/station3.ogg",
  ["station4"] = "assets/sounds/station4.ogg",
  ["station5"] = "assets/sounds/station5.ogg",
  ["station6"] = "assets/sounds/station6.ogg",
  ["station7"] = "assets/sounds/station7.ogg",
  ["station8"] = "assets/sounds/station8.ogg",
  ["station9"] = "assets/sounds/station9.ogg",
  ["station10"] = "assets/sounds/station10.ogg",
  ["heyTaxi"] = "assets/sounds/heytaxi.ogg",
  ["thanks"] = "assets/sounds/thanks.ogg",
  ["thankYou"] = "assets/sounds/thankyou.ogg",
  ["whistle"] = "assets/sounds/whistle.ogg",
}

local getSound = function(name)
  if sound[name] then return love.audio.newSource(sound[name]) end
end


return {
  font = font,
  spritesheet = spritesheet,
  bird = bird,
  taxi = taxi,
  platform = platform,
  passenger = passenger,
  getSound = getSound
}

