
-----------
-- Actor --
-----------

local assets      = require "source.Assets"
local Object      = require "source.system.Object"
local Actor       = Object:extend()
local font_height = assets.font:getHeight()



local addToCollisionWorld = function(self)
  self.collisionWorld = require("source.system.Level"):getCollisionWorld()
  self.collisionWorld:add(self, self.x, self.y, self.width, self.height)
end


local updateCollisionWorld = function(self)
  if not self.collidable then return end
  if self.collisionWorld:hasItem(self) then self.collisionWorld:update(self, self.x, self.y, self.width, self.height)
  else print("Unable to update collision world for Obj ID'"..tostring(self.type)..", "..tostring(self.id).."': object not found!") end
end



function Actor:new(x, y, flags)
  local f = flags or {}
  Actor.super.new(self)
  self:setPosition(x, y)
  self:setOffset(0, 0)
  self:setDimensions()
  self:setColor(255, 255, 255, 255)
  self.visible    = f.visible or true
  self.collidable = f.collidable or true
  self.priority   = 0
  if self.collidable then
    addToCollisionWorld(self)
  end
end


function Actor:setColor(r, g, b, a)
  self.color = {r , g, b, a or self.color[4]}
end


function Actor:setPosition(x, y)
  self.x = x or self.x or 0
  self.y = y or self.y or 0
  updateCollisionWorld(self)
end


function Actor:getPosition()
  return self.x, self.y
end


function Actor:setOffset(x, y)
  self.ox = x or self.ox
  self.oy = y or self.oy
end


function Actor:setPriority(v)
  self.priority = v or self.priority
  local world = require("source.system.Level"):getObjectWorld()
  if world then world:order() end
end


function Actor:setDimensions(width, height)
  self.width  = width or 16
  self.height = height or 16
  updateCollisionWorld(self)
end


function Actor:destroy()
  if self.collidable then self.collisionWorld:remove(self) end
  self.remove = true
  self = nil
end


function Actor:filter(other)
  return "slide"
end


function Actor:processCollisions(x, y)
  local newx, newy = x, y
  local wx, wy, cols, len = self.collisionWorld:move(self, newx, newy, self.filter)
  if len > 0 then 
    for i=1, #cols do
      local other = cols[i].other
      if self.resolveCollision and other.collidable then self:resolveCollision(other) end
    end
      newx = wx
  newy = wy
  end

  return newx, newy
end


function Actor:move(dx, dy)
  local newx, newy = self.x + dx, self.y + dy
  if self.x == newx and self.y == newy then return end
  if self.yvel ~= 0 then self.onGround = false end
  if self.collidable then newx, newy = self:processCollisions(newx, newy) end
  self.x = newx
  self.y = newy
end


function Actor:addAnimation(name, animation)
  if not self.animations then 
    self.animations = {}
  end
  self.currentAnimation = animation:clone()
  self.animations[name] = self.currentAnimation
  return self.currentAnimation
end



function Actor:setAnimation(name)
  if self.currentAnimation == name then return end
  self.currentAnimation = self.animations[name]
end



function Actor:updateAnimation(dt)
  if self.currentAnimation then self.currentAnimation:update(dt) end
end



function Actor:drawAnimation(spritesheet, ...)
  love.graphics.setColor(self.color)
  if self.currentAnimation and self.visible then
    self.currentAnimation:draw(spritesheet, ...)
  end
  love.graphics.setColor(255, 255, 255, 255)
end


function Actor:drawRectangle(mode)
  love.graphics.setColor(self.color)
  love.graphics.rectangle(mode or "fill", self.x, self.y, self.width, self.height)
  love.graphics.setColor(255, 255, 255, 255)
end


function Actor:drawSprite(...)
  love.graphics.setColor(self.color)
  love.graphics.draw(...)
  love.graphics.setColor(255, 255, 255, 255)
end



function Actor:drawDebugText()
  love.graphics.printf(self.type, self.x-self.width/2, self.y-font_height-2, self.width*2, "center")
  love.graphics.printf("ID: "..self.id, self.x-self.width/2, self.y+self.height+2, self.width*2, "center")
end


function Actor:update(dt)
end


function Actor:draw()
  self:drawRectangle()
  self:drawDebugText()
end

return Actor

