

local config        = require "config"
local Signal        = require "source.library.signal"
local Actor         = require "source.actor.Actor"
local assets        = require "source.Assets"
local StateManager  = require "source.system.StateManager"
local Bird = Actor:extend()
local default_config = {
  xvel = 40,
  yvel = 40,
  timer = 0,
}


--------------------------------------------------------------------------------------------------------- --
-- Local functions --
--------------------------------------------------------------------------------------------------------- --

local getAngle = function(a, b) -- Get the angle of point a to point b --
  return math.atan2(b.x - a.x, b.y - a.y)
end


local getDistance = function(a, b) -- Get the distance between two objects --
  return math.sqrt((a.x - b.x) ^ 2 + (a.y - b.y) ^ 2)
end


local calculateWaypoint = function(self)
  local x = love.math.random(self.width, config._resolutionX -(self.width * 2))
  local y = love.math.random(self.height, config._resolutionY -(self.height * 2))
  self.waypoint     = {x=x, y=y}
  self.angToPoint   = getAngle({x = self.x + self.width / 2, y = self.y + self.height / 2}, self.waypoint)
  self.distToPoint  = getDistance({x = self.x + self.width / 2, y = self.y + self.height / 2}, self.waypoint)
end


local moveToPoint = function(self, dt)
  self.timer = self.timer+dt
  if self.timer > 100 then self.timer = 0 end
  local wavingEffect = math.sin(self.timer*7)*25*dt
  self:move(math.sin(self.angToPoint) * self.xvel * dt, wavingEffect + math.cos(self.angToPoint) * self.yvel * dt)
      
  -- Calculate new distance --
  local newx = self.x + (dt * self.xvel) * math.cos(self.angToPoint)
  local newy = self.y + (dt * self.yvel) * math.sin(self.angToPoint)
  local newdist = getDistance({x = newx, y = newy}, self)
  self.distToPoint = self.distToPoint - newdist
   
  -- Reset state on arrival --
  if self.distToPoint <= 0 then
    calculateWaypoint(self)
    if self.waypoint.x < self.x+(self.width/2) then self.flipped = true
    else self.flipped = false end
  end
end



--------------------------------------------------------------------------------------------------------- --
-- Bird Entity --
--------------------------------------------------------------------------------------------------------- --

function Bird:new(x, y)
  Bird.super.new(self)
  for k,v in pairs(default_config) do self[k] = v end
  self:setOffset(2, 2)
  self:setDimensions(16-self.ox*2, 10-self.oy*2)
  self:setPriority(3)
  self:setPosition(x or love.math.random(0, config._resolutionX-self.width), y or love.math.random(0, config._resolutionY-self.height))
  self:addAnimation("Fly", assets.bird)
  self:setAnimation("Fly")
  self.waypoint = {x=0, y=0}
  self.distToPoint  = 0
  self.angToPoint   = 0
end


function Bird:filter(other)
  if other.type == "Taxi" then return "touch" end
end


function Bird:resolveCollision(other)
  if other.type == "Taxi" then
    other:destroy()
  end
end


function Bird:update(dt)
  moveToPoint(self, dt)
  self:updateAnimation(dt)
end


function Bird:draw()
  if self.flipped then self:drawAnimation(assets.spritesheet, self.x, self.y, 0, -1, 1, self.width+self.ox, self.oy)
  else self:drawAnimation(assets.spritesheet, self.x, self.y, 0, 1, 1, self.ox, self.oy) end
  --love.graphics.circle("fill", self.waypoint.x, self.waypoint.y, 1)
end

return Bird



