

local Object    = require "source.library.classic"


local Container = Object:extend()
local type, pairs, ipairs, t_sort, t_remove = type, pairs, ipairs, table.sort, table.remove

function Container:new()
  self.objects = {}
  return self
end


function Container:add(object)
  object.id = #self.objects+1
  self.objects[object.id] = object
  if object.setPriority then object:setPriority(object.priority or 0) end
  return object
end


function Container:remove(v)
  for i=#self.objects, 1, -1 do
    local object = self.objects[i]
    if object == v then
      object:destroy()
      object = nil
    end
  end
end


function Container:getObjects(type)
  if not type then return self.objects end
  local t = {}
  for _, object in ipairs(self.objects) do
    if object.type and object.type == type then t[#t+1] = object end
  end
  return t
end


function Container:getObject(obj)
  if not obj then return end
  if type(obj) == "string" then
    for _, object in ipairs(self.objects) do
      if object.type and object.type == obj then return object end
    end
  elseif type(obj) == "number" then
    for _, object in ipairs(self.objects) do
      if object._objectID == obj then return object end
    end
  end
end


function Container:clear()
  for _, object in pairs(self.objects) do object:destroy() end
end


function Container:order()
  t_sort(self.objects, function(a, b) return a.priority < b.priority end)
end


function Container:update(dt)
  for i=#self.objects, 1, -1 do
    local object = self.objects[i]
    if not object.remove then
      if object.update then object:update(dt) end
    else
      t_remove(self.objects, i)
      object = nil
    end
  end
end


function Container:draw()
  for _, object in pairs(self.objects) do
    if object.draw then object:draw() end
  end
end

return Container


