

local config      = require "config"
local Timer       = require "source.library.timer"
local Container   = require "source.system.Container"
local Bump        = require "source.library.bump"
local Platform    = require "source.actor.Platform"
local Taxi        = require "source.actor.Taxi"
local Passenger   = require "source.actor.Passenger"
local Bird        = require "source.actor.Bird"


local Level = {}


local getPassengerAmount = function(self, id)
  local passengers  = self.objectWorld:getObjects("Passenger")
  local amount      = 0
  for k, passenger in ipairs(passengers) do
    if passenger.currentStation.stationID == id then
      amount = amount+1
    end
  end
  return amount
end


function Level:init()
  self.timer  = Timer()
  self.world  = Container()
  self.objectWorld = Container()
  self.collisionWorld = Bump.newWorld()
    
  -- Adding Player
  self.player = Taxi()
  self.player:setPosition(config._resolutionX/2-self.player.width/2, config._resolutionY/2-32)
  self.objectWorld:add(self.player)
  
  -- Adding Platforms
  local taxiStation = Platform("TaxiStation", 1)
  taxiStation:setPosition(100-taxiStation.width/2, 48)
  self.objectWorld:add(taxiStation)
  
  taxiStation = Platform("TaxiStation", 2)
  taxiStation:setPosition(config._resolutionX-100-taxiStation.width/2, 48)
  self.objectWorld:add(taxiStation)
  
  taxiStation = Platform("TaxiStation", 3)
  taxiStation:setPosition(config._resolutionX/3-taxiStation.width/2, config._resolutionY/1.25)
  self.objectWorld:add(taxiStation)
  
  taxiStation = Platform("TaxiStation", 4)
  taxiStation:setPosition(config._resolutionX/1.5-taxiStation.width/2, config._resolutionY/1.25)
  self.objectWorld:add(taxiStation)
  
  local gasStation = Platform("GasStation")
  gasStation:setPosition(config._resolutionX/2-taxiStation.width/2, config._resolutionY/2-16)
  self.objectWorld:add(gasStation)
  
  -- Variables
  self.passengersPerStation = 5
  self.maxPassengers = #self:getTaxiStations()*self.passengersPerStation
  
  -- Spawn passengers
  self:addPassenger()
  self.timer:every(8.5, function()self:addPassenger()end)
  
  local bird = Bird()
  self.objectWorld:add(bird)
  
  return self
end


function Level:addPassenger(startStation, endStation)
  local stations    = self:getTaxiStations()
  local start       = startStation or love.math.random(1, #stations)
  local destination = love.math.random(1, #stations)
  if #self.objectWorld:getObjects("Passenger") >= self.maxPassengers then return end
  if #stations <= 1 then return end
  
  -- Select spawnpoint
  while getPassengerAmount(self, stations[start]:getStationID()) >= self.passengersPerStation do
    start = love.math.random(1, #stations)
  end
  -- Select Destination
  while destination == start do
    destination = endStation or love.math.random(1, #stations)
  end
  return self.objectWorld:add(Passenger(stations[start], stations[destination]))
end


function Level:getTaxiStations()
  return self.objectWorld:getObjects("TaxiStation")
end


function Level:getTimer()
  return self.timer
end


function Level:getObjectWorld()
  return self.objectWorld
end


function Level:getCollisionWorld()
  return self.collisionWorld
end


function Level:update(dt)
  self.timer:update(dt)
  self.objectWorld:update(dt)
end


function Level:draw()
  self.objectWorld:draw()
  love.graphics.print("World Objects: "..#self.objectWorld:getObjects(), 10, 20)
  love.graphics.print("Press R to Reset", 10, 30)
end


function Level:keypressed(key)
  self.player:keypressed(key)
  if key == "e" then self:addPassenger() end
end

return Level



