

local Object = require "source.system.Object"
local StateManager = Object:extend()
local function __NULL__() end


function StateManager:new(owner)
  self.states = {}
  self.currentState = {}
  self.owner = owner
  return self
end


function StateManager:add(name, state)
  self.states[name] = state
end


function StateManager:switch(name, ...)
  if not name then self.currentState = nil return end
  local update, draw = self.states[name](self.owner, ...)
  self.currentState.name = name
  self.currentState.update = update or __NULL__
  self.currentState.draw = draw or __NULL__
end


function StateManager:getCurrentState()
  if not self.currentState then return end
  return self.currentState.name
end


function StateManager:update(dt)
  if self.currentState then self.currentState.update(self.owner, dt) end
end


function StateManager:draw()
  if self.currentState then self.currentState.draw(self.owner) end
end

return StateManager

